SystemOrganization addCategory: #'EyeSee-Support'!

!Symbol methodsFor: '*eyesee-support' stamp: 'AndreHora 4/12/2011 15:28'!
eyeSeeValue: anObject 
	^ anObject perform: self! !

!BlockContext methodsFor: '*eyesee-support' stamp: 'AndreHora 4/12/2011 15:28'!
eyeSeeValue: valueOrArray
	self numArgs = 0 ifTrue: [ ^ self value ].  
	self numArgs = 1 ifTrue: [ ^ self value: valueOrArray ].
	
	"if I have more than 1 argument, then the valueOrArray is a collection"
	valueOrArray isCollection ifFalse: [ self error: 'A block with more than one argument can only be evaluated with a collection ' ].
	
	self numArgs = valueOrArray size ifTrue: [ ^ self valueWithArguments: valueOrArray ]. 
	
	self error: 'Incorrect number of arguments'! !

Object subclass: #ESAbstractNormalizer
	instanceVariableNames: 'command'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EyeSee-Support'!

!ESAbstractNormalizer classSide methodsFor: 'instance creation' stamp: 'AndreHora 4/11/2011 23:56'!
new
	"Answer a newly created and initialized instance."
	
	^super new initialize! !

!ESAbstractNormalizer methodsFor: 'accessing' stamp: 'AndreHora 4/11/2011 23:56'!
command
	
	^command! !

!ESAbstractNormalizer methodsFor: 'accessing' stamp: 'AndreHora 4/11/2011 23:56'!
command: aBlock
	
	command := aBlock! !

!ESAbstractNormalizer methodsFor: 'accessing' stamp: 'AndreHora 4/12/2011 15:28'!
eyeSeeValue: anEntity
	
	self subclassResponsibility! !

ESAbstractNormalizer subclass: #ESLinearNormalizer
	instanceVariableNames: 'maximumValue context minimumValue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EyeSee-Support'!

ESLinearNormalizer subclass: #ESColorLinearNormalizer
	instanceVariableNames: 'minBrightness maxBrightness lowColor highColor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EyeSee-Support'!

!ESColorLinearNormalizer classSide methodsFor: 'instance creation' stamp: 'AndreHora 4/11/2011 23:53'!
inContext: aCollection lowColor: lowColor highColor: highColor
	
	| normalizer |
	normalizer := self inContext: aCollection.
	normalizer lowColor: lowColor.
	normalizer highColor: highColor.
	^normalizer! !

!ESColorLinearNormalizer classSide methodsFor: 'instance creation' stamp: 'AndreHora 4/11/2011 23:53'!
inContext: aCollection lowColor: lowColor lowThreshold: lowThreshold highColor: highColor highThreshold: highThreshold
	
	| normalizer |
	normalizer := self inContext: aCollection.
	normalizer lowColor: lowColor.
	normalizer highColor: highColor.
	normalizer minBrightness: lowThreshold.
	normalizer maxBrightness: highThreshold.
	^normalizer! !

!ESColorLinearNormalizer classSide methodsFor: 'instance creation' stamp: 'AndreHora 4/11/2011 23:53'!
inContext: aCollection withCommand: aBlock lowColor: lowColor highColor: highColor
	
	| normalizer |
	normalizer := self inContext: aCollection withCommand: aBlock.
	normalizer lowColor: lowColor.
	normalizer highColor: highColor.
	^normalizer! !

!ESColorLinearNormalizer classSide methodsFor: 'instance creation' stamp: 'AndreHora 4/11/2011 23:53'!
inContext: aCollection withCommand: aBlock lowColor: lowColor lowThreshold: lowThreshold highColor: highColor highThreshold: highThreshold
	
	| normalizer |
	normalizer := self inContext: aCollection withCommand: aBlock.
	normalizer lowColor: lowColor.
	normalizer highColor: highColor.
	normalizer minBrightness: lowThreshold.
	normalizer maxBrightness: highThreshold.
	^normalizer! !

!ESColorLinearNormalizer methodsFor: 'private' stamp: 'AndreHora 4/11/2011 23:53'!
blend: aColor with: aColorValue weight: wt
	"Answer a new ColorValue which is a weighted blend of the receiver and the
	supplied ColorValue."
	"Simply interpolates in RGB space."

	| rv gv bv invWt |
	
	invWt := 1-wt.
	rv := ((aColor red * invWt) + ((aColorValue red)*wt)) "rounded".
	gv := ((aColor green  * invWt) + ((aColorValue green)*wt)) "rounded".
	bv := ((aColor blue *invWt)+ ((aColorValue blue)*wt)) "rounded".
	^Color r: rv g: gv b: bv! !

!ESColorLinearNormalizer methodsFor: 'accessing' stamp: 'AndreHora 4/12/2011 15:28'!
eyeSeeValue: anEntity
	"Calculates the color of the entity based on the context."
	
	| myValue myHighColor myLowColor myMinBrightness myMaxBrightness weight |
	"Get color specific data"
	myMinBrightness := self minBrightnessFor: anEntity.
	myMaxBrightness := self maxBrightnessFor: anEntity.
	myHighColor := self highColorFor: anEntity.
	myLowColor := self lowColorFor: anEntity.	"Get data"
	myValue := (self command value: anEntity) - (self minimumValue: anEntity).
	weight := myMinBrightness
		+
			(myValue abs / ((self maximumValue: anEntity) - (self minimumValue: anEntity))
				* (myMaxBrightness - myMinBrightness)).
	^self blend: myLowColor with: myHighColor weight: weight! !

!ESColorLinearNormalizer methodsFor: 'accessing' stamp: 'AndreHora 4/11/2011 23:53'!
highColor: aBlock
	
	highColor := aBlock! !

!ESColorLinearNormalizer methodsFor: 'translator protocol' stamp: 'AndreHora 4/12/2011 15:28'!
highColorFor: anEntity
	
	^highColor eyeSeeValue: anEntity! !

!ESColorLinearNormalizer methodsFor: 'initialize-release' stamp: 'AndreHora 4/11/2011 23:53'!
initialize
	
	super initialize.
	lowColor := Color white.
	highColor := Color black! !

!ESColorLinearNormalizer methodsFor: 'accessing' stamp: 'AndreHora 4/11/2011 23:53'!
lowColor: aBlock
	
	lowColor := aBlock! !

!ESColorLinearNormalizer methodsFor: 'translator protocol' stamp: 'AndreHora 4/12/2011 15:28'!
lowColorFor: anEntity
	
	^lowColor eyeSeeValue: anEntity! !

!ESColorLinearNormalizer methodsFor: 'accessing' stamp: 'AndreHora 4/11/2011 23:53'!
maxBrightness: aBlock
	
	maxBrightness := aBlock! !

!ESColorLinearNormalizer methodsFor: 'translator protocol' stamp: 'AndreHora 4/12/2011 15:28'!
maxBrightnessFor: anEntity
	
	^maxBrightness isNil
		ifTrue: [1.0]
		ifFalse: [maxBrightness eyeSeeValue: anEntity]! !

!ESColorLinearNormalizer methodsFor: 'accessing' stamp: 'AndreHora 4/11/2011 23:53'!
minBrightness: aBlock
	
	minBrightness := aBlock! !

!ESColorLinearNormalizer methodsFor: 'translator protocol' stamp: 'AndreHora 4/12/2011 15:28'!
minBrightnessFor: anEntity
	
	^minBrightness isNil
		ifTrue: [0.0]
		ifFalse: [minBrightness eyeSeeValue: anEntity]! !

!ESLinearNormalizer classSide methodsFor: 'instance creation' stamp: 'AndreHora 4/11/2011 23:53'!
inContext: aCollection
	
	^self inContext: aCollection withCommand: #yourself! !

!ESLinearNormalizer classSide methodsFor: 'instance creation' stamp: 'AndreHora 4/11/2011 23:53'!
inContext: aCollection withCommand: aBlock
	
	^(self new)
		context: aCollection;
		command: aBlock;
		yourself! !

!ESLinearNormalizer methodsFor: 'accessing' stamp: 'AndreHora 4/11/2011 23:53'!
command: aBlock
	
	super command: aBlock.
	self flushCache! !

!ESLinearNormalizer methodsFor: 'accessing' stamp: 'AndreHora 4/11/2011 23:53'!
context
	
	^context! !

!ESLinearNormalizer methodsFor: 'accessing' stamp: 'AndreHora 4/11/2011 23:53'!
context: aContext
	
	context := aContext.
	self flushCache! !

!ESLinearNormalizer methodsFor: 'accessing' stamp: 'AndreHora 4/11/2011 23:53'!
flushCache
	
	maximumValue := nil! !

!ESLinearNormalizer methodsFor: 'initialize-release' stamp: 'AndreHora 4/11/2011 23:53'!
initialize
	"Initialize a newly created instance. This method must answer the receiver."
	
	super initialize.
	maximumValue := nil! !

!ESLinearNormalizer methodsFor: 'private' stamp: 'AndreHora 4/12/2011 15:28'!
maximumValue: anEntity
	"Returns the largest value that the color metric provides"
	
	maximumValue isNil
		ifTrue:
			[maximumValue := (self context eyeSeeValue: anEntity)
				inject: 1
				into:
					[:maximum :entity | maximum max: (self command eyeSeeValue: entity)]].
	^maximumValue! !

!ESLinearNormalizer methodsFor: 'private' stamp: 'AndreHora 4/12/2011 15:28'!
minimumValue: anEntity
	"Returns the smallest value that the color metric provides"
	
	minimumValue isNil
		ifTrue:
			[minimumValue := (self context eyeSeeValue: anEntity)
				inject: Float infinity 
				into:
					[:minimum :entity | minimum min: (self command eyeSeeValue: entity)]].
	^minimumValue! !

Object subclass: #ESMalMatrix
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EyeSee-Support'!

ESMalMatrix subclass: #ESMalArrayMatrix
	instanceVariableNames: 'array columnSize rowSize'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EyeSee-Support'!

!ESMalArrayMatrix methodsFor: 'accessing' stamp: 'AndreHora 4/11/2011 23:39'!
at: row and: column
	
	^array at: (row - 1) * self columnSize + column! !

!ESMalArrayMatrix methodsFor: 'accessing' stamp: 'AndreHora 4/11/2011 23:39'!
at: row and: column put: aNumber
	
	^array
		at: (row - 1) * self columnSize + column
		put: aNumber! !

!ESMalArrayMatrix methodsFor: 'private' stamp: 'SergeStinckwich 12/8/2011 15:50'!
checkInvariant
	^ array isCollection and: [ array size = (rowSize * columnSize) ]! !

!ESMalArrayMatrix methodsFor: 'accessing' stamp: 'AndreHora 4/11/2011 23:39'!
columnSize
	
	^columnSize! !

!ESMalArrayMatrix methodsFor: 'initialize-release' stamp: 'AndreHora 4/11/2011 23:39'!
initializeRows: n columns: m
	
	rowSize := n.
	columnSize := m.
	array := Array new: n * m.
	^self! !

!ESMalArrayMatrix methodsFor: 'accessing' stamp: 'AndreHora 4/11/2011 23:39'!
rowSize
	
	^rowSize! !

!ESMalMatrix classSide methodsFor: 'accessing' stamp: 'AndreHora 4/11/2011 23:37'!
browserIcon
"Answer icon to be displayed with matrix classes."
	
	^ nil! !

!ESMalMatrix classSide methodsFor: 'instance creation' stamp: 'TudorGirba 11/16/2011 10:11'!
new
	"Create a new matrix with dimension zero."
	
	^self new: 0! !

!ESMalMatrix classSide methodsFor: 'instance creation' stamp: 'AndreHora 4/11/2011 23:37'!
new: numberOrPoint
	"Create a new square matrix with the given dimension."
	
	| point |
	point := numberOrPoint asPoint.
	^self
		rows: point x
		columns: point y! !

!ESMalMatrix classSide methodsFor: 'instance creation' stamp: 'AndreHora 4/11/2011 23:37'!
new: numberOrPoint withAll: value
	
	^(self new: numberOrPoint)
		atAllPut: value;
		yourself! !

!ESMalMatrix classSide methodsFor: 'instance creation' stamp: 'AndreHora 4/11/2011 23:37'!
readFromDenseText: aStream
	
	| m n matrix tmp |
	m := Integer readFrom: aStream skipSeparators.
	n := Integer readFrom: aStream skipSeparators.
	matrix := self rows: m columns: n.
	1
		to: m
		do:
			[:row | 
			1
				to: n
				do:
					[:column | 
					matrix
						at: row
						and: column
						"put: (tmp := (Number readSmalltalkAndCSyntaxFrom: aStream skipSeparators))."
						put: (tmp := (Number readFrom: aStream skipSeparators)).
						
						]].
	^matrix! !

!ESMalMatrix classSide methodsFor: 'instance creation' stamp: 'AndreHora 4/11/2011 23:37'!
readFromSparseText: aStream
	
	| m n matrix |
	n := Integer readFrom: aStream skipSeparators.
	m := Integer readFrom: aStream skipSeparators.
	Integer readFrom: aStream skipSeparators.
	matrix := self
		rows: m
		columns: n
		withAll: Number zero.
	1
		to: m
		do:
			[:rows | 
			1
				to: (Integer readFrom: aStream skipSeparators)
				do:
					[:unused | 
					matrix
						at: rows
						and: (Integer readFrom: aStream skipSeparators) + 1
						put: (Number readFrom: aStream skipSeparators)]].
	^matrix! !

!ESMalMatrix classSide methodsFor: 'instance creation' stamp: 'AndreHora 4/11/2011 23:37'!
rows: m columns: n
	"Create a new matrix with m rows and n columns."
	
	^(self basicNew)
		initializeRows: m columns: n;
		yourself! !

!ESMalMatrix classSide methodsFor: 'instance creation' stamp: 'AndreHora 4/11/2011 23:37'!
rows: m columns: n withAll: value
	
	^(self rows: m columns: n)
		atAllPut: value;
		yourself! !

!ESMalMatrix classSide methodsFor: 'private' stamp: 'AndreHora 4/11/2011 23:37'!
switch: anObject caseMatrix: matrixBlock caseNestedCollection: nestedColletionBlock caseCollection: collectionBlock default: defaultBlock
	
	(anObject isKindOf: MalMatrix) ifTrue: [^matrixBlock value].
	(anObject isKindOf: Collection)
		ifTrue:
			[(anObject isEmpty not
				and:
					[anObject
						allSatisfy:
							[:each | (each isKindOf: Collection) and: [each size = anObject first size]]])
				ifTrue: [^nestedColletionBlock value]
				ifFalse: [^collectionBlock value]].
	^defaultBlock value! !

!ESMalMatrix classSide methodsFor: 'constants access' stamp: 'AndreHora 4/11/2011 23:37'!
unity
	
	^MalMatrixCoercion with: 1! !

!ESMalMatrix classSide methodsFor: 'private' stamp: 'AndreHora 4/11/2011 23:37'!
vectorSpecies
	
	^MalArrayVector! !

!ESMalMatrix classSide methodsFor: 'instance creation' stamp: 'AndreHora 4/11/2011 23:37'!
withAll: object
	
	| dimension |
	dimension := self
		switch: object
		caseMatrix: [object dimension]
		caseNestedCollection: [object size @ object first size]
		caseCollection: [1 @ object size]
		default: [0].
	^(self new: dimension)
		atAllPut: object;
		yourself! !

!ESMalMatrix classSide methodsFor: 'constants access' stamp: 'AndreHora 4/11/2011 23:37'!
zero
	
	^MalMatrixCoercion with: 0! !

!ESMalMatrix methodsFor: 'arithmetic' stamp: 'AndreHora 4/11/2011 23:37'!
* aNumber
	
	^aNumber productFromMatrix: self! !

!ESMalMatrix methodsFor: 'arithmetic' stamp: 'AndreHora 4/11/2011 23:37'!
*= aNumber
	
	^self
		with: (self coerce: aNumber)
		affect: [:a :b | a * b]! !

!ESMalMatrix methodsFor: 'matrix functions' stamp: 'AndreHora 4/11/2011 23:37'!
*~ aMatrix
	"Answer the matrix multiplication of the reciever with the argument."
	
	| newMatrix |
	self columnSize = aMatrix rowSize ifFalse: [Array new noMatchError].
	newMatrix := self species
		rows: self rowSize
		columns: aMatrix columnSize.	"workaround: this should be done in a more generic way with double dispatch."
	newMatrix
		putWithIndices:
			((aMatrix isKindOf: MalDiagonalMatrix)
				ifTrue: [[:i :j | (self at: i and: j) * (aMatrix at: j and: j)]]
				ifFalse:
					[[:i :j | 
					(1 to: self columnSize)
						inject: 0
						into:
							[:sum :k | sum + ((self at: i and: k) * (aMatrix at: k and: j))]]]).
	^newMatrix! !

!ESMalMatrix methodsFor: 'arithmetic' stamp: 'AndreHora 4/11/2011 23:37'!
+ aNumber
	
	^aNumber sumFromMatrix: self! !

!ESMalMatrix methodsFor: 'arithmetic' stamp: 'AndreHora 4/11/2011 23:37'!
+= aNumber
	
	^self
		with: (self coerce: aNumber)
		affect: [:a :b | a + b]! !

!ESMalMatrix methodsFor: 'arithmetic' stamp: 'AndreHora 4/11/2011 23:37'!
- aNumber
	
	^aNumber differenceFromMatrix: self! !

!ESMalMatrix methodsFor: 'arithmetic' stamp: 'AndreHora 4/11/2011 23:37'!
-= aNumber
	
	^self
		with: (self coerce: aNumber)
		affect: [:a :b | a - b]! !

!ESMalMatrix methodsFor: 'arithmetic' stamp: 'AndreHora 4/11/2011 23:37'!
/ aNumber
	
	^aNumber quotientFromMatrix: self! !

!ESMalMatrix methodsFor: 'arithmetic' stamp: 'AndreHora 4/11/2011 23:37'!
/= aNumber
	
	^self
		with: (self coerce: aNumber)
		affect: [:a :b | a / b]! !

!ESMalMatrix methodsFor: 'comparing' stamp: 'AndreHora 4/11/2011 23:37'!
= matrix
	"Answer whether the dimension of the receiver is the same as otherMatrix' dimension,
	and each of the receiver's elements equal the corresponding element of otherMatrix."
	
	^self == matrix
		or:
			[(matrix isKindOf: MalMatrix)
				and:
					[(matrix hasDimension: self dimension)
						and:
							[self
								with: matrix
								do: [:a :b | a = b ifFalse: [^false]].
							true]]]! !

!ESMalMatrix methodsFor: 'arithmetic' stamp: 'AndreHora 4/11/2011 23:37'!
adaptToNumber: rcvr andSend: selector

	^ self perform: selector with: rcvr! !

!ESMalMatrix methodsFor: 'arithmetic' stamp: 'AndreHora 4/11/2011 23:37'!
adaptToPoint: rcvr andSend: selector

	^ self perform: selector with: rcvr! !

!ESMalMatrix methodsFor: 'resizing' stamp: 'AndreHora 4/11/2011 23:37'!
addColumn
	"Add a column. Must implement if isExtendable returns true."
	
	self shouldNotImplement! !

!ESMalMatrix methodsFor: 'resizing' stamp: 'AndreHora 4/11/2011 23:37'!
addColumnBeforeIndex: anIndex
	"Insert a column before anIndex. Must implement if isResizebale returns true."
	
	self shouldNotImplement! !

!ESMalMatrix methodsFor: 'resizing' stamp: 'AndreHora 4/11/2011 23:37'!
addRow
	"Add a row. Must implement if isExtendable returns true."
	
	self shouldNotImplement! !

!ESMalMatrix methodsFor: 'resizing' stamp: 'AndreHora 4/11/2011 23:37'!
addRowBeforeIndex: anIndex
	"Insert a row before anIndex. Must implement if isResizebale returns true."
	
	self shouldNotImplement! !

!ESMalMatrix methodsFor: 'converting' stamp: 'AndreHora 4/11/2011 23:37'!
asArray
	
	| array stream |
	array := Array new: self rowSize * self columnSize.
	stream := WriteStream on: array.
	self do: [:each | stream nextPut: each].
	^array! !

!ESMalMatrix methodsFor: 'accessing' stamp: 'AndreHora 4/11/2011 23:37'!
at: row and: column
	"Answer the vakue at the given indices."
	
	^self subclassResponsibility! !

!ESMalMatrix methodsFor: 'arithmetic' stamp: 'AndreHora 4/11/2011 23:37'!
at: row and: column add: aNumber
	"Add aNumber to the element at row and column.  Answer the sum."
	
	^self
		at: row
		and: column
		put: (self at: row and: column) + aNumber! !

!ESMalMatrix methodsFor: 'arithmetic' stamp: 'AndreHora 4/11/2011 23:37'!
at: row and: column divide: aNumber
	"Divide the element at row and column by aNumber.  Answer the quotient."
	
	^self
		at: row
		and: column
		put: (self at: row and: column) / aNumber! !

!ESMalMatrix methodsFor: 'arithmetic' stamp: 'AndreHora 4/11/2011 23:37'!
at: row and: column multiply: aNumber
	"Multiply aNumber with the element at row and column.  Answer the product."
	
	^self
		at: row
		and: column
		put: (self at: row and: column) * aNumber! !

!ESMalMatrix methodsFor: 'accessing' stamp: 'AndreHora 4/11/2011 23:37'!
at: row and: column put: aNumber
	"Store the vakue at the given indices."
	
	^self subclassResponsibility! !

!ESMalMatrix methodsFor: 'arithmetic' stamp: 'AndreHora 4/11/2011 23:37'!
at: row and: column subtract: aNumber
	"Subtract aNumber from the element at row and column.  Answer the difference."
	
	^self
		at: row
		and: column
		put: (self at: row and: column) - aNumber! !

!ESMalMatrix methodsFor: 'accessing' stamp: 'AndreHora 4/11/2011 23:37'!
atAllPut: object
	
	self class
		switch: object
		caseMatrix:
			[self
				with: object
				affect: [:a :b | b]]
		caseNestedCollection:
			[object size = self rowSize
				ifTrue:
					[[object first size = self columnSize] assert.
					self putWithIndices: [:row :column | (object at: row) at: column]]
				ifFalse:
					[object size = self columnSize
						ifTrue:
							[[object first size = self rowSize] assert.
							self putWithIndices: [:row :column | (object at: column) at: row]]
						ifFalse: [Error raise]]]
		caseCollection:
			[| stream |
			[object size = (self rowSize * self columnSize)] assert.
			stream := ReadStream on: object.
			self
				withIndices:
					[:row :column | 
					self
						at: row
						and: column
						put: stream next]]
		default:
			[self withIndices: [:row :column | self at: row and: column put: object]].
	^object! !

!ESMalMatrix methodsFor: 'accessing' stamp: 'AndreHora 4/11/2011 23:37'!
atPoint: rowColumnCoordinate
	
	^self
		at: rowColumnCoordinate x
		and: rowColumnCoordinate y! !

!ESMalMatrix methodsFor: 'accessing' stamp: 'AndreHora 4/11/2011 23:37'!
atPoint: rowColumnCoordinate put: aNumber
	
	^self
		at: rowColumnCoordinate x
		and: rowColumnCoordinate y
		put: aNumber! !

!ESMalMatrix methodsFor: 'private' stamp: 'AndreHora 4/11/2011 23:37'!
checkInvariant
	
	^self subclassResponsibility! !

!ESMalMatrix methodsFor: 'coercing' stamp: 'AndreHora 4/11/2011 23:37'!
coerce: aNumber
	
	^MalMatrixCoercion with: aNumber! !

!ESMalMatrix methodsFor: 'enumerating' stamp: 'AndreHora 4/11/2011 23:37'!
collect: aBlock
	
	| newMatrix |
	newMatrix := self copyEmpty.
	self
		withIndices:
			[:row :column | 
			newMatrix
				at: row
				and: column
				put: (aBlock value: (self at: row and: column))].
	^newMatrix! !

!ESMalMatrix methodsFor: 'accessing vectorwise' stamp: 'AndreHora 4/11/2011 23:37'!
columnAt: columnIndex
	
	^MalMatrixPath column: columnIndex on: self! !

!ESMalMatrix methodsFor: 'accessing vectorwise' stamp: 'AndreHora 4/11/2011 23:37'!
columnAt: columnIndex put: vector
	
	vector doWithIndex: [:each :row | self at: row and: columnIndex put: each].
	^vector! !

!ESMalMatrix methodsFor: 'accessing' stamp: 'AndreHora 4/11/2011 23:37'!
columnSize
	
	^self subclassResponsibility! !

!ESMalMatrix methodsFor: 'accessing vectorwise' stamp: 'AndreHora 4/11/2011 23:37'!
columns
	
	| array |
	array := Array new: self columnSize.
	1
		to: self columnSize
		do:
			[:index | 
			array
				at: index
				put: (self columnAt: index)].
	^array! !

!ESMalMatrix methodsFor: 'copying' stamp: 'AndreHora 4/11/2011 23:37'!
copyEmpty
	"Answer a copy of the receiver that contains no elements."
	
	^self copyEmpty: self dimension! !

!ESMalMatrix methodsFor: 'copying' stamp: 'AndreHora 4/11/2011 23:37'!
copyEmpty: dimension
	"Answer a copy of the receiver that contains no elements."
	
	^self species new: dimension! !

!ESMalMatrix methodsFor: 'copying' stamp: 'AndreHora 4/11/2011 23:37'!
deepCopy
	
	^self species withAll: self! !

!ESMalMatrix methodsFor: 'accessing' stamp: 'AndreHora 4/11/2011 23:37'!
density
	
	^self sparseSize / self size! !

!ESMalMatrix methodsFor: 'arithmetic' stamp: 'AndreHora 4/11/2011 23:37'!
determinant
	
	| a11 a12 a21 a22 |
	[self hasDimension: 2 @ 2] assert.
	a11 := self atPoint: 1 @ 1.
	a12 := self atPoint: 1 @ 2.
	a21 := self atPoint: 2 @ 1.
	a22 := self atPoint: 2 @ 2.
	^a11 * a22 - (a12 * a21)! !

!ESMalMatrix methodsFor: 'accessing vectorwise' stamp: 'AndreHora 4/11/2011 23:37'!
diagonal
	
	^MalMatrixPath diagonalOn: self! !

!ESMalMatrix methodsFor: 'double dispatching' stamp: 'AndreHora 4/11/2011 23:37'!
differenceFromDouble: aNumber
	
	^self differenceFromNumber: aNumber! !

!ESMalMatrix methodsFor: 'double dispatching' stamp: 'AndreHora 4/11/2011 23:37'!
differenceFromFixedPoint: aNumber
	
	^self differenceFromNumber: aNumber! !

!ESMalMatrix methodsFor: 'double dispatching' stamp: 'AndreHora 4/11/2011 23:37'!
differenceFromFloat: aNumber
	
	^self differenceFromNumber: aNumber! !

!ESMalMatrix methodsFor: 'double dispatching' stamp: 'AndreHora 4/11/2011 23:37'!
differenceFromFraction: aNumber
	
	^self differenceFromNumber: aNumber! !

!ESMalMatrix methodsFor: 'double dispatching' stamp: 'AndreHora 4/11/2011 23:37'!
differenceFromInteger: aNumber
	
	^self differenceFromNumber: aNumber! !

!ESMalMatrix methodsFor: 'double dispatching' stamp: 'AndreHora 4/11/2011 23:37'!
differenceFromMatrix: aMatrix
	
	^aMatrix
		with: self
		collect: [:a :b | a - b]! !

!ESMalMatrix methodsFor: 'double dispatching' stamp: 'AndreHora 4/11/2011 23:37'!
differenceFromNumber: aNumber
	
	^self collect: [:b | aNumber - b]! !

!ESMalMatrix methodsFor: 'double dispatching' stamp: 'AndreHora 4/11/2011 23:37'!
differenceFromPoint: aNumber
	
	^self differenceFromNumber: aNumber! !

!ESMalMatrix methodsFor: 'double dispatching' stamp: 'AndreHora 4/11/2011 23:37'!
differenceFromSmallDouble: aNumber
	
	^self differenceFromNumber: aNumber! !

!ESMalMatrix methodsFor: 'accessing' stamp: 'AndreHora 4/11/2011 23:37'!
dimension
	
	^self rowSize @ self columnSize! !

!ESMalMatrix methodsFor: 'enumerating' stamp: 'AndreHora 4/11/2011 23:37'!
do: aBlock
	
	self
		withIndices: [:row :column | aBlock value: (self at: row and: column)].
	^self! !

!ESMalMatrix methodsFor: 'enumerating' stamp: 'AndreHora 4/11/2011 23:37'!
doSparseWithIndices: eachRowColumnBlock
	
	self
		doWithIndices:
			[:each :row :column | 
			each isZero
				ifFalse: [eachRowColumnBlock value: each value: row value: column]].
	^self! !

!ESMalMatrix methodsFor: 'enumerating' stamp: 'AndreHora 4/11/2011 23:37'!
doWithIndices: eachRowColumnBlock
	
	self
		withIndices:
			[:row :column | 
			eachRowColumnBlock
				value: (self at: row and: column)
				value: row
				value: column]! !

!ESMalMatrix methodsFor: 'accessing' stamp: 'AndreHora 4/11/2011 23:37'!
first
	
	^self at: 1 and: 1! !

!ESMalMatrix methodsFor: 'coercing' stamp: 'AndreHora 4/11/2011 23:37'!
generality
	
	^240! !

!ESMalMatrix methodsFor: 'testing' stamp: 'AndreHora 4/11/2011 23:37'!
hasDimension: aDimension
	
	^self dimension = aDimension! !

!ESMalMatrix methodsFor: 'comparing' stamp: 'TudorGirba 11/16/2011 10:17'!
hash
	
	^(self dimension hash bitXor: (self atPoint: 1 @ 1) hash)
		bitXor: (self atPoint: self dimension) hash! !

!ESMalMatrix methodsFor: 'initialize-release' stamp: 'AndreHora 4/11/2011 23:37'!
initializeRows: m columns: n
	
	^self subclassResponsibility! !

!ESMalMatrix methodsFor: 'enumerating' stamp: 'AndreHora 4/11/2011 23:37'!
inject: value into: aBlock
	
	| result |
	result := value.
	self do: [:each | result := aBlock value: result value: each].
	^result! !

!ESMalMatrix methodsFor: 'testing' stamp: 'AndreHora 4/11/2011 23:37'!
isComplex
	
	^false! !

!ESMalMatrix methodsFor: 'testing' stamp: 'AndreHora 4/11/2011 23:37'!
isDiagonal
	"Answer if the reciever is a diagonal matrix."
	
	self isSquare ifFalse: [^false].
	self
		doSparseWithIndices:
			[:value :row :column | row = column ifFalse: [^false]].
	^true! !

!ESMalMatrix methodsFor: 'testing' stamp: 'AndreHora 4/11/2011 23:37'!
isResizeable
	
	^false! !

!ESMalMatrix methodsFor: 'testing' stamp: 'AndreHora 4/11/2011 23:37'!
isSquare
	"Answer if the reciever is a square matrix."
	
	^self rowSize = self columnSize! !

!ESMalMatrix methodsFor: 'testing' stamp: 'AndreHora 4/11/2011 23:37'!
isSymetric
	"Answer if the reciever is a symetric matrix."
	
	self isSquare ifFalse: [^false].
	1
		to: self columnSize
		do:
			[:column | 
			1
				to: column
				do:
					[:row | (self at: column and: row) = (self at: row and: column) ifFalse: [^false]]].
	^true! !

!ESMalMatrix methodsFor: 'testing' stamp: 'AndreHora 4/11/2011 23:37'!
isZero
	"Answer if the reciever contains no non-zero numbers."
	
	self doSparseWithIndices: [:value :row :column | ^false].
	^true! !

!ESMalMatrix methodsFor: 'testing' stamp: 'AndreHora 4/11/2011 23:37'!
isZeroAt: row and: column
	
	^(self at: row and: column) isZero! !

!ESMalMatrix methodsFor: 'accessing' stamp: 'AndreHora 4/11/2011 23:37'!
last
	
	^self
		at: self rowSize
		and: self columnSize! !

!ESMalMatrix methodsFor: 'private' stamp: 'AndreHora 4/11/2011 23:37'!
noMatchError
	
	Error raise! !

!ESMalMatrix methodsFor: 'private' stamp: 'AndreHora 4/11/2011 23:37'!
noModificationError
	
	Error raise! !

!ESMalMatrix methodsFor: 'printing' stamp: 'AndreHora 4/11/2011 23:37'!
printOn: aStream
	"Append to the argument aStream a sequence of characters that identifies the matrix."
	
	| tooMany |
	tooMany := aStream position + 5000.
	aStream
		print: self class;
		space;
		print: self dimension;
		space;
		nextPut: $(.
	(1 to: self rowSize)
		do:
			[:row | 
			aStream nextPut: $(.
			(1 to: self columnSize)
				do:
					[:column | 
					aStream position > tooMany
						ifTrue:
							[aStream nextPutAll: (#etc << #dialogs >> '...etc...') asString , '))'.
							^self].
					aStream print: (self at: row and: column)]
				separatedBy: [aStream space].
			aStream nextPut: $)]
		separatedBy: [aStream space].
	aStream nextPut: $)! !

!ESMalMatrix methodsFor: 'double dispatching' stamp: 'AndreHora 4/11/2011 23:37'!
productFromDouble: aNumber
	
	^self productFromNumber: aNumber! !

!ESMalMatrix methodsFor: 'double dispatching' stamp: 'AndreHora 4/11/2011 23:37'!
productFromFixedPoint: aNumber
	
	^self productFromNumber: aNumber! !

!ESMalMatrix methodsFor: 'double dispatching' stamp: 'AndreHora 4/11/2011 23:37'!
productFromFloat: aNumber
	
	^self productFromNumber: aNumber! !

!ESMalMatrix methodsFor: 'double dispatching' stamp: 'AndreHora 4/11/2011 23:37'!
productFromFraction: aNumber
	
	^self productFromNumber: aNumber! !

!ESMalMatrix methodsFor: 'double dispatching' stamp: 'AndreHora 4/11/2011 23:37'!
productFromInteger: aNumber
	
	^self productFromNumber: aNumber! !

!ESMalMatrix methodsFor: 'double dispatching' stamp: 'AndreHora 4/11/2011 23:37'!
productFromMatrix: aMatrix
	
	^aMatrix
		with: self
		collect: [:a :b | a * b]! !

!ESMalMatrix methodsFor: 'double dispatching' stamp: 'AndreHora 4/11/2011 23:37'!
productFromNumber: aNumber
	
	^self collect: [:b | aNumber * b]! !

!ESMalMatrix methodsFor: 'double dispatching' stamp: 'AndreHora 4/11/2011 23:37'!
productFromPoint: aNumber
	
	^self productFromNumber: aNumber! !

!ESMalMatrix methodsFor: 'double dispatching' stamp: 'AndreHora 4/11/2011 23:37'!
productFromSmallDouble: aNumber
	
	^self productFromNumber: aNumber! !

!ESMalMatrix methodsFor: 'enumerating' stamp: 'AndreHora 4/11/2011 23:37'!
putWithIndices: rowColumnBlock
	
	self
		withIndices:
			[:row :column | 
			self
				at: row
				and: column
				put: (rowColumnBlock value: row value: column)].
	^self! !

!ESMalMatrix methodsFor: 'double dispatching' stamp: 'AndreHora 4/11/2011 23:37'!
quotientFromDouble: aNumber
	
	^self quotientFromNumber: aNumber! !

!ESMalMatrix methodsFor: 'double dispatching' stamp: 'AndreHora 4/11/2011 23:37'!
quotientFromFixedPoint: aNumber
	
	^self quotientFromNumber: aNumber! !

!ESMalMatrix methodsFor: 'double dispatching' stamp: 'AndreHora 4/11/2011 23:37'!
quotientFromFloat: aNumber
	
	^self quotientFromNumber: aNumber! !

!ESMalMatrix methodsFor: 'double dispatching' stamp: 'AndreHora 4/11/2011 23:37'!
quotientFromFraction: aNumber
	
	^self quotientFromNumber: aNumber! !

!ESMalMatrix methodsFor: 'double dispatching' stamp: 'AndreHora 4/11/2011 23:37'!
quotientFromInteger: aNumber
	
	^self quotientFromNumber: aNumber! !

!ESMalMatrix methodsFor: 'double dispatching' stamp: 'AndreHora 4/11/2011 23:37'!
quotientFromMatrix: aMatrix
	
	^aMatrix
		with: self
		collect: [:a :b | a / b]! !

!ESMalMatrix methodsFor: 'double dispatching' stamp: 'AndreHora 4/11/2011 23:37'!
quotientFromNumber: aNumber
	
	^self collect: [:b | aNumber / b]! !

!ESMalMatrix methodsFor: 'double dispatching' stamp: 'AndreHora 4/11/2011 23:37'!
quotientFromPoint: aNumber
	
	^self quotientFromNumber: aNumber! !

!ESMalMatrix methodsFor: 'double dispatching' stamp: 'AndreHora 4/11/2011 23:37'!
quotientFromSmallDouble: aNumber
	
	^self quotientFromNumber: aNumber! !

!ESMalMatrix methodsFor: 'arithmetic' stamp: 'AndreHora 4/11/2011 23:37'!
reciprocal
	
	^self
		collect:
			[:each | 
			each isZero
				ifTrue: [each]
				ifFalse: [each reciprocal]]! !

!ESMalMatrix methodsFor: 'converting' stamp: 'AndreHora 4/11/2011 23:37'!
remap: mapping
	
	^MalMappedMatrix on: self rowMap: mapping columnMap: mapping! !

!ESMalMatrix methodsFor: 'resizing' stamp: 'AndreHora 4/11/2011 23:37'!
removeColumnAtIndex: anIndex
	
	self shouldNotImplement! !

!ESMalMatrix methodsFor: 'resizing' stamp: 'AndreHora 4/11/2011 23:37'!
removeRowAtIndex: anIndex
	
	self shouldNotImplement! !

!ESMalMatrix methodsFor: 'testing' stamp: 'AndreHora 4/11/2011 23:37'!
respondsToArithmetic
	"We are arithmetic, yes."
	
	^true! !

!ESMalMatrix methodsFor: 'arithmetic' stamp: 'AndreHora 4/11/2011 23:37'!
roundTo: aNumber
	
	^self collect: [:each | each roundTo: aNumber]! !

!ESMalMatrix methodsFor: 'accessing vectorwise' stamp: 'AndreHora 4/11/2011 23:37'!
rowAt: rowIndex
	
	^MalMatrixPath row: rowIndex on: self! !

!ESMalMatrix methodsFor: 'accessing vectorwise' stamp: 'AndreHora 4/11/2011 23:37'!
rowAt: rowIndex put: vector
	
	vector
		doWithIndex: [:each :column | self at: rowIndex and: column put: each].
	^vector! !

!ESMalMatrix methodsFor: 'accessing' stamp: 'AndreHora 4/11/2011 23:37'!
rowSize
	
	^self subclassResponsibility! !

!ESMalMatrix methodsFor: 'accessing vectorwise' stamp: 'AndreHora 4/11/2011 23:37'!
rows
	
	| array |
	array := Array new: self rowSize.
	1
		to: self rowSize
		do:
			[:index | 
			array
				at: index
				put: (self rowAt: index)].
	^array! !

!ESMalMatrix methodsFor: 'accessing' stamp: 'AndreHora 4/11/2011 23:37'!
size
	
	^self rowSize * self columnSize! !

!ESMalMatrix methodsFor: 'accessing' stamp: 'AndreHora 4/11/2011 23:37'!
sparseSize
	
	| tally |
	tally := 0.
	self do: [:each | each isZero ifTrue: [tally := tally + 1]].
	^tally! !

!ESMalMatrix methodsFor: 'private' stamp: 'AndreHora 4/11/2011 23:37'!
species
	
	^self class! !

!ESMalMatrix methodsFor: 'printing' stamp: 'AndreHora 4/11/2011 23:37'!
storeDenseTextOn: aStream
	"Append to the argument aStream a sequence of characters 
	that identifies the collection."
	
	aStream
		print: self rowSize;
		space;
		print: self columnSize;
		cr.
	1
		to: self rowSize
		do:
			[:row | 
			(1 to: self columnSize)
				do: [:column | aStream print: (self at: row and: column)]
				separatedBy: [aStream space].
			aStream cr]! !

!ESMalMatrix methodsFor: 'printing' stamp: 'AndreHora 4/11/2011 23:37'!
storeSparseTextOn: aStream
	"Append to the argument aStream a sequence of characters 
	that identifies the collection."
	
	aStream
		print: self columnSize;
		space;
		print: self rowSize;
		space;
		print: self sparseSize;
		cr.
	self rows
		do:
			[:row | 
			aStream
				print: row sparseSize;
				cr.
			row
				doSparseWithIndex:
					[:value :index | 
					aStream
						print: index - 1;
						space;
						print: value;
						cr]]! !

!ESMalMatrix methodsFor: 'double dispatching' stamp: 'AndreHora 4/11/2011 23:37'!
sumFromDouble: aNumber
	
	^self sumFromNumber: aNumber! !

!ESMalMatrix methodsFor: 'double dispatching' stamp: 'AndreHora 4/11/2011 23:37'!
sumFromFixedPoint: aNumber
	
	^self sumFromNumber: aNumber! !

!ESMalMatrix methodsFor: 'double dispatching' stamp: 'AndreHora 4/11/2011 23:37'!
sumFromFloat: aNumber
	
	^self sumFromNumber: aNumber! !

!ESMalMatrix methodsFor: 'double dispatching' stamp: 'AndreHora 4/11/2011 23:37'!
sumFromFraction: aNumber
	
	^self sumFromNumber: aNumber! !

!ESMalMatrix methodsFor: 'double dispatching' stamp: 'AndreHora 4/11/2011 23:37'!
sumFromInteger: aNumber
	
	^self sumFromNumber: aNumber! !

!ESMalMatrix methodsFor: 'double dispatching' stamp: 'AndreHora 4/11/2011 23:37'!
sumFromMatrix: aMatrix
	
	^aMatrix
		with: self
		collect: [:a :b | a + b]! !

!ESMalMatrix methodsFor: 'double dispatching' stamp: 'AndreHora 4/11/2011 23:37'!
sumFromNumber: aNumber
	
	^self collect: [:b | aNumber + b]! !

!ESMalMatrix methodsFor: 'double dispatching' stamp: 'AndreHora 4/11/2011 23:37'!
sumFromPoint: aNumber
	
	^self sumFromNumber: aNumber! !

!ESMalMatrix methodsFor: 'double dispatching' stamp: 'AndreHora 4/11/2011 23:37'!
sumFromSmallDouble: aNumber
	
	^self sumFromNumber: aNumber! !

!ESMalMatrix methodsFor: 'matrix functions' stamp: 'AndreHora 4/11/2011 23:37'!
transposed
	
	^MalMatrixView transposeOn: self! !

!ESMalMatrix methodsFor: 'private' stamp: 'AndreHora 4/11/2011 23:37'!
vectorSpecies
	
	^self class vectorSpecies! !

!ESMalMatrix methodsFor: 'enumerating' stamp: 'AndreHora 4/11/2011 23:37'!
with: aMatrix affect: eachOtherBlock
	
	[aMatrix hasDimension: self dimension] assert.
	self
		putWithIndices:
			[:row :column | 
			eachOtherBlock
				value: (self at: row and: column)
				value: (aMatrix at: row and: column)]! !

!ESMalMatrix methodsFor: 'enumerating' stamp: 'AndreHora 4/11/2011 23:37'!
with: aMatrix collect: eachOtherBlock
	
	| newMatrix |
	[aMatrix hasDimension: self dimension] assert.
	newMatrix := self copyEmpty: self dimension.
	newMatrix
		putWithIndices:
			[:row :column | 
			eachOtherBlock
				value: (self at: row and: column)
				value: (aMatrix at: row and: column)].
	^newMatrix! !

!ESMalMatrix methodsFor: 'enumerating' stamp: 'AndreHora 4/11/2011 23:37'!
with: aMatrix do: eachOtherBlock
	
	[aMatrix hasDimension: self dimension] assert.
	self
		withIndices:
			[:row :column | 
			eachOtherBlock
				value: (self at: row and: column)
				value: (aMatrix at: row and: column)]! !

!ESMalMatrix methodsFor: 'enumerating' stamp: 'AndreHora 4/11/2011 23:37'!
withIndices: aBlock
	
	1
		to: self rowSize
		do:
			[:row | 
			1
				to: self columnSize
				do: [:column | aBlock value: row value: column]]! !

!Object methodsFor: '*eyesee-support' stamp: 'AndreHora 4/12/2011 15:28'!
eyeSeeValue: anArgument 
	^ self! !

!BlockClosure methodsFor: '*eyesee-support' stamp: 'AndreHora 4/12/2011 15:28'!
eyeSeeValue: valueOrArray
	self numArgs = 0 ifTrue: [ ^ self value ].  
	self numArgs = 1 ifTrue: [ ^ self value: valueOrArray ].
	
	"if I have more than 1 argument, then the valueOrArray is a collection"
	valueOrArray isCollection ifFalse: [ self error: 'A block with more than one argument can only be evaluated with a collection ' ].
	
	self numArgs = valueOrArray size ifTrue: [ ^ self valueWithArguments: valueOrArray ]. 
	
	self error: 'Incorrect number of arguments'! !
